import {BillDetailItems} from "@/components/bill";
import {Button, Col, Form, Modal, Row, Space, Toast} from "@douyinfe/semi-ui";
import React from "react";
import {useTranslation} from "react-i18next";
import {finishBill} from "@/service/api/bill.ts";
import {BizError} from "@/service/types.ts";
import {useSetState} from "ahooks";

type BillPaidModalProps = {
	bill: BillModel;
	open?: boolean;
	onConfirm: () => void
	onCancel: () => void
}
export const BillPaidModal: React.FC<BillPaidModalProps> = (props) => {
	const {t} = useTranslation()

	const [state, setState] = useSetState<{
		loading?: boolean
	}>({})
	const onSubmit = (values: BillUpdateParams) => {
		if (!props.bill) return;
		setState({
			loading: true
		})
		finishBill({bill: props.bill, param: values})
			.then(() => {
				setState({loading: false})
				props.onConfirm()
			})
			.catch((e: BizError) => {
				setState({loading: false})
				Toast.error({
					content: `${t('base.operate_fail')}:${e.message}`,
					duration: 3
				})
			})
	}
	return <Modal
		title={t('bill.set_bill_paid') + ` - ${props.bill?.id}`}
		visible={props.open}
		closeOnEsc={true}
		onCancel={props.onCancel}
		footer={null}
		width={600}
		okText={t('base.confirm_paid')}
		maskClosable={false}
	>
		{props.bill && <div><BillDetailItems bill={props.bill}/></div>}

		<Form<BillUpdateParams> onSubmit={onSubmit} initValues={{
			payment_channel: 'FLYWIRE',
			payment_method: '',
			merchant_ref: props.bill?.merchant_ref,
			payment_amount: props.bill?.amount,
			actual_payment_amount: props.bill?.amount
		}}>
			<Row gutter={20}>
				<Col span={12}>
					<Form.Select
						rules={[
							{required: true, message: 'required error'},
						]}
						showClear field="payment_channel" label={t('bill.title_pay_channel')}
						placeholder={t('base.please_select')} style={{width: '100%'}}>
						<Form.Select.Option value="FLYWIRE">FLYWIRE</Form.Select.Option>
						{/*<Form.Select.Option value="ASIAPAY">ASIAPAY</Form.Select.Option>*/}
						{/*<Form.Select.Option value="PPS">PPS</Form.Select.Option>*/}
					</Form.Select>
				</Col>
				<Col span={12}>
					<Form.Select
						rules={[
							{required: true, message: 'required error'},
						]}
						optionList={[
							{value: 'card', label: 'Card(VISA,MasterCard,UnionPay,JCB...)'},
							{value: 'wechat', label: 'Wechat'},
							{value: 'alipay', label: 'Alipay'},
							{value: 'other', label: 'Other'},
						]}
						allowCreate filter showClear field="payment_method" label={t('bill.title_pay_method')}
						placeholder={t('base.please_select')} style={{width: '100%'}}/>
				</Col>
			</Row>
			<Row gutter={20}>
				<Col span={12}>
					<Form.Input
						rules={[
							{required: true, message: 'required error'},
						]}
						showClear field="merchant_ref" label="Merchant Ref"
						placeholder={t('base.please_enter')} style={{width: '100%'}}/>
				</Col>
				<Col span={6}>
					<Form.Input
						rules={[
							{required: true, message: 'required error'},
						]} type={'number'} showClear
						field="payment_amount" label={t('bill.title_pay_amount')}
						placeholder={t('base.please_enter')} style={{width: '100%'}}/>
				</Col>
				<Col span={6}>
					<Form.Input
						type={'number'}
						showClear field="actual_payment_amount" label={t('bill.title_actual_payment_amount')}
						placeholder={t('base.please_enter')} style={{width: '100%'}}/>
				</Col>
			</Row>
			<Row gutter={20}>
				<Col span={24}>
					<Form.TextArea
						rules={[{required: true, message: 'required error'}]}
						showClear field="remark" label={t('bill.title_remark')} placeholder={t('base.please_enter')}
						style={{width: '100%'}}/>
				</Col>
			</Row>
			{/*<p style={{marginTop: 10}}>{t('bill.paid_confirm')}</p>*/}
			<div className={'text-right'} style={{margin: '10px 0 20px'}}>
				<Space spacing={12}>
					<Button onClick={props.onCancel} type={'tertiary'}>{t('base.cancel')}</Button>
					<Button
						loading={state.loading} htmlType={'submit'} theme={'solid'}
						type={'primary'}>{t('base.confirm_paid')}</Button>
				</Space>
			</div>
		</Form>
	</Modal>
}