import {Button, Select, Space, Divider, InputNumber, Modal} from "@douyinfe/semi-ui";
import React, {useEffect} from "react";
import {useSetState} from "ahooks";
import MoneyFormat from "@/components/money-format.tsx";
import {useTranslation} from "react-i18next";
import {useBillTypes} from "@/hooks/useBillTypes.ts";
import {NumberConfirm} from "@/pages/bill/components/number_confirm.tsx";
import {BillDetailItems} from "@/components/bill";
import {BillDetailItem} from "@/components/bill/bill-detail-items.tsx";
import {IconMoney, IconStudentId} from "@/components/icons";
import {confirmBillType} from "@/service/api/bill.ts";

type BillTypeConfirmProps = {
	bill: BillModel;
	onClose?: (refresh?: boolean) => void;
	onChange?: (confirms: ConfirmedBillDetail[]) => void;
}

export const BillTypeConfirm: React.FC<BillTypeConfirmProps> = (props) => {
	const {t} = useTranslation()
	const confirmed: ConfirmedBillDetail[] = props.bill.details.map(it=>({
		bill_type: it.bill_type,
        bill_detail_id: it.id,
        amount: Number(it.amount)
	}))
	const [state, setState] = useSetState({
		confirm_application_number: '', confirmed
	})
	const BillTypes = useBillTypes()

	const onChange = (value: string, index: number, type: 'type' | 'amount') => {
		if (state.confirmed.length <= index || !value) return;
		const confirmed = [...state.confirmed]
		if (type == 'type') {
			confirmed[index].bill_type = value
		} else {
			confirmed[index].amount = Number(value)
		}

		setState({confirmed})
		props.onChange?.(confirmed)
	}

	const addOrRemove = (index:number)=>{
		// 不允许删除最后一个
		if (index > -1 && state.confirmed.length <= 1) return;
		const confirmed = [...state.confirmed,...(index==-1?[{bill_type: '', amount: 0, bill_detail_id: 0}]:[])]
		if(index > -1) confirmed.splice(index, 1)
		setState({confirmed})
		props.onChange?.(confirmed)
	}
	useEffect(()=>{
		props.onChange?.(confirmed)
	},[])



	return (<>
		<Divider>Bill Type Confirm</Divider>
		{
			state.confirmed.map((item, index) => (
				<div key={index} className="confirm-item-btn align-center space-between" style={{marginTop: 20}}>
					<Select
						value={item.bill_type}
						style={{width: 240}}
						onChange={v => onChange(String(v), index, 'type')}
						placeholder={t('base.please_select_bill_type')}>
						{
							BillTypes.map((it, idx) => (
								<Select.Option key={idx} value={it.label}>{it.label}</Select.Option>))
						}
					</Select>

					<Space spacing={10}>
						<InputNumber
							hideButtons precision={2} value={item.amount} type={'number'}
							onChange={v => onChange(String(v), index, 'amount')} style={{width: 140}}/>
						<Button
							disabled={state.confirmed.length <= 1} onClick={() => addOrRemove(index)}
							theme={'solid'} type={'secondary'}>{t('base.remove')}</Button>
					</Space>
				</div>
			))
		}
		<div style={{marginTop: 10,marginBottom:20}}>
			<Button onClick={()=>addOrRemove(-1)}>{t('base.add')}</Button>
		</div>
		<Divider />
	</>)
}

export const BillTypeConfirmModal: React.FC<BillTypeConfirmProps> = (props) => {
	const {t} = useTranslation()
	const [state, setState] = useSetState<{
		confirm_application_number: string;
		detail_confirms: ConfirmedBillDetail[];
		loading?: boolean;
	}>({
		confirm_application_number: '',
		detail_confirms: []
	})

	const onBillConfirm = () => {
		// 判断confirm的总金额是否和实付金额相等
		const total = state.detail_confirms.reduce((total, item) => {
            return total + Number(item.amount)
        }, 0)
		if(total != props.bill.actual_payment_amount){
			Modal.warning({
                title: 'Warning',
                content: t('bill.confirm_bill_warning_amount')
            })
            return;
		}


		setState({loading: true})
		confirmBillType([{
			id: props.bill.id,
			confirm_student_number: props.bill.student_number,
			...state
		}]).then(() => {
			props.onClose?.(true)
		}).catch(e=>{
			Modal.error({
				title: 'Error',
				content: `Confirmed Fail: ${e.message}`
			})
		}).finally(() => {
			setState({loading: false})
		})
	}
	return (<Modal
		title={t('bill.confirm_bill')}
		visible={true}
		closeOnEsc={true}
		width={550}
		footer={null}
		onCancel={() => props.onClose?.()}
	>
		<div>
			<BillDetailItems bill={props.bill} studentNumberRender={<>
				<BillDetailItem
					icon={<IconStudentId/>} title={t('manual.student_number')}
					value={props.bill.student_number || '-'}/>
				<BillDetailItem
					icon={<IconStudentId/>} title={t('base.bill_number')}
					value={props.bill.application_number || '-'}/>
			</>}/>
			<BillDetailItem
				icon={<IconMoney/>} title={t('bill.title_actual_payment_amount')}
				value={<MoneyFormat money={props.bill.actual_payment_amount}/>}/>
		</div>
		<div className="confirm-number-container" style={{padding: '15px 0'}}>
			<Divider>Bill Number Confirm</Divider>
			{/*{*/}
			{/*	!state.confirmBill.student_number_confirm &&*/}
			{/*    <NumberConfirm bill={state.confirmBill} type={'student_number'}/>*/}
			{/*}*/}
			<NumberConfirm
				onChange={confirm_application_number => setState({confirm_application_number})}
				bill={props.bill} type={'application_number'}/>
		</div>
		<BillTypeConfirm bill={props.bill} onChange={detail_confirms => setState({detail_confirms})}/>
		<div className={'text-center'} style={{paddingBottom: 20,marginTop:20}}>
			<Button
				onClick={onBillConfirm} loading={state.loading} type={'primary'}
				theme={'solid'}>{t('base.confirm')}</Button>
		</div>
	</Modal>)
}