import {Button, Space, TabPane, Tabs, Popconfirm, Toast, Modal} from "@douyinfe/semi-ui";
import {useRequest, useSetState} from "ahooks";
import {useTranslation} from "react-i18next";
import {useState} from "react";

import SearchForm from "@/components/bill/search-form.tsx";
import {BillList} from "@/components/bill/list.tsx";
import {billList, BillQueryParams, confirmBills} from "@/service/api/bill.ts";
import {BizError} from "@/service/types.ts";

const BillReconciliation = () => {
	const {t} = useTranslation()
	const [queryParams, setBillQueryParams] = useState<BillQueryParams>({
		apply_status: 'UNCHECKED'
	});
	const {data, loading, refresh} = useRequest(() => billList({
		...queryParams,
		status: 'PAID',
		confirm_status: 'CONFIRMED'
	}), {
		refreshDeps: [queryParams],
		onSuccess: () => {
			document.documentElement.scrollTo({top:0});
			setState({checkingId: -1})
		},
		onError: (e: Error) => {
			Toast.error({
				content: `${t('base.query_bill')}:${e.message}`,
				duration: 3
			})
		}
	})

	const [selectKeys, setSelectedKeys] = useState<(string | number)[]>([])
	const [state, setState] = useSetState({
		checkingId: -1
	})
	const confirmBill = (records: number[]) => {
		if (records.length == 0) {
			Toast.error({content: t('bill.confirm_select_empty')})
			return
		}
		const arr = data?.list.filter(item => records.includes(item.id));
		if (!arr) return;
		for (let i = 0; i < arr.length; i++) {
			const item = arr[i];

			// 判断confirm的总金额是否和实付金额相等
			const total = item.detail_confirms ? item.detail_confirms.reduce((total, item) => {
				return total + Number(item.amount)
			}, 0) : 0;

			if (total != item.actual_payment_amount) {
				Modal.warning({
					title: 'Warning',
					content: t('bill.confirm_bill_warning_amount_id', {id: item.id})
				})
				return;
			}
		}

		setState({checkingId: records.length > 1 ? 0 : Number(records[0])})
		confirmBills(records).then(() => {
			Toast.success({content: t('bill.confirm_success')})
			refresh()
		}).catch((e: BizError) => {
			Toast.error({
				content: `${t('base.operate_fail')}:${e.message}`,
				duration: 3
			})
		}).finally(() => {
			setState({checkingId: -1})
		})
	}
	const operation = (_record: BillModel) => {
		return (<Space>
			<Popconfirm
				title={'Notice'}
				content={t('bill.confirm_confirm_title')}
				onConfirm={() => confirmBill([_record.id])}
				okText={t('base.confirm')}
				cancelText={t('base.cancel')}
				disabled={state.checkingId == _record.id}
			>
				<Button
					loading={state.checkingId == _record.id} size={'small'} theme={'solid'}
					type={'primary'}>{t('bill.confirm')}</Button>
			</Popconfirm>
		</Space>)
	}

	return (<div>

		<SearchForm
			searchHeader={(
				<Tabs className={'no-border'} onChange={(apply_status) => setBillQueryParams({
					apply_status,
					status: 'PAID'
				})}>
					<TabPane tab={<span>{t('bill.reconciliation_status_pending')}</span>} itemKey="UNCHECKED"/>
					<TabPane tab={<span>{t('bill.reconciliation_status_submitted')}</span>} itemKey="CHECKED"/>
				</Tabs>
			)}
			loading={loading}
			onSearch={(params) => {
				setBillQueryParams({...params, apply_status: queryParams.apply_status})
			}}
		/>
		<BillList
			source={data} type={'reconciliation'}
			operationRender={queryParams.apply_status == 'CHECKED' ? undefined : operation}
			beforeTotalAmount={<div>{queryParams.apply_status != 'CHECKED' && (
				(selectKeys.length == 0) ? <Button theme={'solid'} disabled style={{marginRight: 10}}>
						{t('bill.confirm_batch')}
					</Button> :
					<Popconfirm
						title={'Notice'}
						content={`${t('bill.cancel_confirm_bills')}?`}
						onConfirm={() => confirmBill(selectKeys as number[])}
					>
						<Button theme={'solid'} style={{marginRight: 10}}>
							{t('bill.confirm_batch')}
						</Button>
					</Popconfirm>
			)}</div>}
			onRowSelection={queryParams.apply_status == 'CHECKED' ? undefined : (keys: (number | string)[]) => {
				setSelectedKeys(keys);
			}}
			loading={loading}
			onPageChange={(page_number) => setBillQueryParams({
				...queryParams,
				page_number
			})}
			onPageSizeChange={(page_size) => {
				setBillQueryParams({
					...queryParams,
					page_size
				})
			}}
		/>
	</div>)
}
export default BillReconciliation