import {useTranslation} from "react-i18next";
import {Button} from "@douyinfe/semi-ui";
import {useParams, useSearchParams} from "react-router-dom";

import styles from "@/pages/pay/pay.module.less";
import {PayLogo} from "@/pages/pay/component";
import {SuccessIcon} from "@/assets/images/pay/success.tsx";
import {useDownloadReceiptPDF} from "@/service/generate-pdf.ts";
import {useEffect} from "react";
import {useSetState} from "ahooks";
import {FailIcon} from "@/assets/images/pay/fail.tsx";
import {getBillDetail, updateBillPaymentSuccess} from "@/service/api/bill.ts";
import {BillStatus} from "@/service/types.ts";
import {IconLoading} from "@/components/icons";

type PayResult = 'fail' | 'success' | 'cancel' | 'error' | 'loading' | string;

const QUERY_MAX_COUNT = 50,
    QUERY_DELAY = 6000;

const PayIndex = () => {
    const {t} = useTranslation()
    const {downloadPDF} = useDownloadReceiptPDF()
    const [state, setState] = useSetState<{
        from?: 'ASIAPAY' | 'FLYWIRE' | string;
        result?: PayResult;
        id?: string;
        bill?: BillModel
        status?: string | null;
        loading?: boolean;
        queryCount: number;
    }>({
        result: 'loading',
        queryCount: 0
    });

    const param = useParams<{ result: PayResult }>();
    const [search] = useSearchParams(); // 参数有: from: asia_spay || flywire
    const updateBillSuccess = (billId: number, type: string, ref: string) => {
        setState({result: 'loading'})
        updateBillPaymentSuccess(billId, ref, type).then(bill => {
            setState({
                result: 'success',
                bill: bill
            })
        }).catch(() => {
            setState({result: 'fail'})
        })
    }
    const checkBillSuccess = (billId: number) => {
        setState({
            result: 'loading',
            queryCount: state.queryCount + 1
        })

        getBillDetail(Number(billId)).then((bill) => {
            // 判断bill状态
            if (bill.status != BillStatus.PAID) {
                if(state.queryCount > QUERY_MAX_COUNT) {
                    setState({result: 'fail'})
                }else{
                    // 重试
                    setTimeout(()=>checkBillSuccess(billId), QUERY_DELAY);
                }
                return;
            }
            setState({
                result: 'success',
                bill: bill
            })
        }).catch(() => {
        })
    }
    useEffect(() => {
        const result = param.result,
            from = search.get('from'),
            status = search.get('status')
        if (from) {
            setState({from})
            const bill = search.get('bill')
            // 参数无效
            if (!bill) {
                setState({result: 'fail'})
                return;
            }
            if (result == 'success') {
                if (from == 'ASIAPAY') {
                    updateBillSuccess(Number(bill), from, search.get('Ref')!);
                } else if (from == 'FLYWIRE') {
                    checkBillSuccess(Number(bill));
                }
            }
            return;
        }
        setState({result, status})
    }, []);

    return (<div className={styles.container}>
        <PayLogo/>
        {state.result == 'success' ? <>
                <div className={styles.paySuccess}>
                    <div className={styles.successIcon}>
                        <SuccessIcon/>
                    </div>
                    <h2 className={`${styles.payText} pay-text`}>
                        {t('pay.text_success')}
                    </h2>
                </div>
                {state.bill && <div className={styles.payConfirm}>
                    <div
                        className="student-email">{t('bill.bill_number')}: {state.bill.application_number || state.bill.student_number}</div>
                    <div className="pay-submit">
                        <Button
                            theme={'solid'}
                            onClick={() => downloadPDF(state.bill!)}
                            style={{width: 250, marginTop: 20}}>{t('bill.download_receipt')}</Button>
                    </div>
                </div>}
            </>
            : (
                state.result == 'loading' ? <div className={styles.payLoading}>
                    <IconLoading size={70} />
                    <div className={styles.loadingText}>{t('pay.query_pay_status')}</div>
                    </div>
                    : (<div className={styles.paySuccess}>
                        <div className={styles.successIcon} style={{color: '#f05672'}}>
                            <FailIcon/>
                        </div>
                        <h2 className={`${styles.payText} pay-text`}>
                            {state.result == 'fail' ? t('pay.text_failed') : (
                                state.result == 'cancel' ? t('pay.text_canceled') : (t('pay.bill_error'))
                            )}
                        </h2>
                    </div>)
            )
        }
    </div>);
}
export default PayIndex;