import {Outlet, useLocation} from "react-router-dom";
import React, {useMemo} from "react";
import {Avatar, Button, Dropdown, Layout, Nav, Space, Typography} from "@douyinfe/semi-ui"
import {useTranslation} from "react-i18next";

import AuthGuard from "@/routes/layout/auth-guard.tsx";
import AppLogo from "@/assets/AppLogo.tsx";
import useAuth from "@/hooks/useAuth.ts";
import {I18nSwitcher} from "@/i18n";
import {AllDashboardMenu, DashboardNavigation} from "@/routes/layout/dashboard-navigation.tsx";
import {IconExit, IconUser} from "@douyinfe/semi-icons";
import styled from "@emotion/styled";
import useConfig from "@/hooks/useConfig.ts";
import {IconRoles} from "@/components/icons";
import {setCurrentRole} from "@/contexts/auth";

const {Header, Content, Sider} = Layout;

type CommonHeaderProps = {
    children?: React.ReactNode;
    title?: React.ReactNode;
    rightExtra?: React.ReactNode;
}
const HeaderUserProfile = styled.div({
    padding: '10px 20px 0',
    minWidth: 150,
    fontSize: 12,
    lineHeight: 1,
    cursor: 'default',
})
export const HeaderUserAvatar = () => {
    const {t} = useTranslation()
    const {user, logout} = useAuth()
    // const UserAvatar = () => (<Avatar color="orange" size="small">{user?.username.substring(0, 3)}</Avatar>)
    return (<Dropdown
        position={'bottomRight'}
        trigger={'click'}
        render={
            <>
                <HeaderUserProfile>
                    <Space>
                        <Avatar color="orange" size="default"><IconUser /></Avatar>
                        <div>
                            <Typography.Title heading={6}>{user?.username}</Typography.Title>
                            <div style={{maxWidth:250,overflow:'hidden',whiteSpace:'nowrap',textOverflow:'ellipsis'}}>
                                <Typography.Text
                                    type="quaternary"
                                    size={'small'}>Department:{user?.department?.toUpperCase() || "N/A"}</Typography.Text>
                            </div>
                            <div>
                                <Typography.Text
                                        type="quaternary"
                                        size={'small'}>Role:{user?.role?.toUpperCase()}</Typography.Text>
                            </div>
                        </div>
                    </Space>
                </HeaderUserProfile>
                <Dropdown.Menu>
                    <Dropdown.Divider/>
                    <Dropdown.Item icon={<IconExit/>} onClick={logout}>{t('layout.logout')}</Dropdown.Item>
                </Dropdown.Menu>
            </>
        }
    >

        <Avatar color="orange" size="small"><IconUser /></Avatar>
    </Dropdown>)
}
const RoleList: UserRole[] = ['root', 'ro', 'fo','staff']
const RoleSwitcher = ()=>{
    const {user, updateUser} = useAuth()
    // update user role
    const changeRole = (role:UserRole)=>{
        updateUser({role}).then(()=>{
            setCurrentRole(role)
        })
    }
    return (<>
        {user?.origin_role == 'root' && (<Dropdown
            clickToHide
            render={
                <Dropdown.Menu>
                    {RoleList.map((key) => (
                        <Dropdown.Item
                            active={user?.role == key} key={key}
                            onClick={() => changeRole(key)}
                        ><span>{key.toUpperCase()}</span></Dropdown.Item>
                    ))}
                </Dropdown.Menu>
            }
        >
            <Button theme="borderless">
                <Space style={{transform:'translateY(3px)'}}>
                    <IconRoles size={20} color={'white'} />
                    <span style={{color:'white'}}>{user?.role?.toUpperCase()}</span>
                </Space>
            </Button>
        </Dropdown>) }
    </>)
}
export const CommonHeader: React.FC<CommonHeaderProps> = ({children, title, rightExtra}) => {
    const {appName} = useConfig()

    return (<Header style={{position: 'sticky', top: 0, zIndex: 100}}>
        <div>
            <Nav mode="horizontal" defaultSelectedKeys={['Home']}
                 style={{backgroundColor: '#43ABFF', color: '#fff', height: "var(--dashboard-header-height,50px)"}}>
                <Nav.Header>
                    <Space>
                        <AppLogo style={{fontSize: 12, color: 'white'}} theme={'color'}/>
                        {appName}
                    </Space>
                </Nav.Header>
                <div className={'align-center'} style={{color: 'var(--semi-color-text-2)',}}>
                    <span
                        style={{
                            marginRight: '24px',
                            color: 'var(--semi-color-text-0)',
                            fontWeight: '600',
                            fontSize: '1.2em'
                        }}
                    >{title}</span>
                    {children}
                </div>
                <Nav.Footer>
                    <Space>
                        {rightExtra}
                        <RoleSwitcher />
                        <I18nSwitcher/>
                        <HeaderUserAvatar/>
                    </Space>

                </Nav.Footer>
            </Nav>
        </div>
    </Header>)
}


type LayoutProps = {
    children: React.ReactNode
}

const LayoutContentContainer = styled.div({
    borderRadius: 10,
    marginTop: 20
})
export const BaseLayout: React.FC<LayoutProps> = ({children}) => {
    const {t} = useTranslation()
    // get current route path to display menu name
    const location = useLocation();
    const currentRoutePath = useMemo(() => {
        const nav = AllDashboardMenu.find(it => it.path == location.pathname)
        return nav?.key
    }, [location.pathname])

    return (<Layout className={'dashboard-layout'}>
        <CommonHeader/>
        <Layout style={{
            backgroundColor: '#f8f9fa',
            minHeight: 'calc(100vh - var(--dashboard-header-height,50px))',
        }}>
            <Sider style={{minWidth: 'var(--dashboard-navigation-width,250px)',transition:'all 0.5s'}}>
                <DashboardNavigation/>
            </Sider>
            <Content style={{padding: 'var(--dashboard-layout-padding)'}}>
                <Typography.Title>
                    {currentRoutePath ? t(`layout.menu.${currentRoutePath}`).toUpperCase() : ''}
                </Typography.Title>
                <LayoutContentContainer className="content-container">
                    {children}
                </LayoutContentContainer>
            </Content>
        </Layout>
    </Layout>)
}


const DashboardLayout: React.FC<{ children?: React.ReactNode }> = ({children}) => {
    return <AuthGuard>
        <BaseLayout>
            {children ? children : <Outlet/>}
        </BaseLayout>
    </AuthGuard>
}
export default DashboardLayout