import {get, post, put} from "@/service/request.ts";
import dayjs from "dayjs";
import {getAuthToken} from "@/hooks/useAuth.ts";
import {stringify} from "qs";

export type BillQueryResult = {
    result: BillModel[];
    total_count: number;
    query_total_count: number;
    query_total_amount: number;
}
export type BillQueryParams = Partial<BillQueryParam>;
const DEFAULT_PAGE_SIZE = 10;

function formatBillQueryResult(params: BillQueryParams, result: BillQueryResult) {
    // 将 BillQueryResult 转成成 RecordList
    const formatData: RecordList<BillModel> = {
        list: result.result,
        pagination: {
            total: result.query_total_count,
            pageSize: params.page_size || DEFAULT_PAGE_SIZE,
            current: params.page_number || 1,
            recordTotal: Number(result.query_total_amount || 0)
        }
    }
    return formatData
}

export async function billList(params: BillQueryParams) {
    const result = await get<BillQueryResult>('/bills', params)
    return formatBillQueryResult(params, result);
}

export function exportBillList(params: BillQueryParams) {
    const token = getAuthToken();
    const headers = new Headers({
        'Token': token || ''
    });

    return fetch(`${AppConfig.API_PREFIX || '/api'}/bills/export?${stringify(params)}`,{
        headers,
        method:'GET'
    }).then(r=>r.blob())
    // return get<Blob>('/bills/export', params,true)
}
//现场支付创建账单接口
export function createManualBill(params: ManualCreateBillParam) {
    return post<BillModel>('/manual_payment', params)
}

export function selectBillTypeList(){
    return get<BillType[]>('/billing_types')
}

// 获取账单详情
export function getBillDetail(id: number) {
    return get<BillModel>('/bills/' + id)
}

export function getAsiaPayData(id: number) {
    return get<AsiaPayModel>(`/bills/${id}/asiapay`)
}

export function getFlywirePayUrl(id: number, return_cta: string, return_cta_name: string) {
    return post<{ url: string }>(`/bills/${id}/flywire`, {return_cta, return_cta_name})
}

// 作废订单
export function modifyBillStatus(id: number,status: BillStatus) {
    return put(`/bills/${id}/cancel`,{status})
}

export function confirmBillType(bills:BillConfirmParams[]) {
    return post<BillModel>(`/bills/confirm`, {bills})
}

export function confirmBills(bill_ids: number[] | string[]) {
    return post(`/bills/apply`, {bill_ids})
}

export function updateBillPaymentSuccess(bill_id: number, merchant_ref: string, payment_channel: string) {
    return post<BillModel>(`/bills/finish`, {merchant_ref, payment_channel, bill_id})
}


export function createExternalBill(params: ExternalCreateParamsType) {
    return post<BillModel>('/bill', params)
}

type BillUpdateFormParams = {
    bill:BillModel;
    param:BillUpdateParams
}
export async function finishAsiapay({param}: BillUpdateFormParams){
    const paramUrl = `?prc=0&src=0&Ord=12345678&Ref=${param.merchant_ref}&PayRef=123456&successcode=0&Amt=10.00&Cur=344&Holder=Test Card&AuthId=123456&AlertCode=&remark=
&eci=07&payerAuth=U&sourceIp=192.1.1.1&ipCountry=HK&payMethod=VISA
x&cardIssuingCountry=HK&channelType=SPC&`
    const ret = await post<string>(`/flywire/feedback?${paramUrl}`, {},true)

    return ret?.toLowerCase() == 'ok'
}

export async function finishFlywire({bill,param}: BillUpdateFormParams){
    const eventData = {
        "event_type": "guaranteed",
        "event_date": dayjs().format('YYYY-MM-DDTHH:mm:ss[Z]'),
        "event_resource": "payments",
        "data": {
            "remark": param.remark,
            "payment_id": param.merchant_ref,
            "amount_from": Number(param.actual_payment_amount) * 100,
            "currency_from": "HKD",
            "amount_to": Number(param.payment_amount) * 100,
            "currency_to": "HKD",
            "status": "guaranteed",
            "expiration_date": dayjs().format('YYYY-MM-DDTHH:mm:ss[Z]'),
            "external_reference": bill.id,
            "country": "CN",
            "payment_method": {
                "type": param.payment_method
            },
            "fields": {
                "student_email": bill.student_email,
                "student_last_name": bill.student_last_name || bill.student_english_name,
                "student_id": bill.student_number,
                "resident_no": null,
                "contact_no": "",
                "payment_type_other": null,
                "payment_type": null
            }
        }
    }
    const ret = await post<string>('/flywire/feedback', eventData,true)

    return ret?.toLowerCase() == 'ok'
}

export function finishBill(params: BillUpdateFormParams) {
    if(params.param.payment_channel === 'flywire'){
        return finishAsiapay(params)
    }
    return finishFlywire(params)
}